


VERIFY_ENTITIES = {
    "IN": ["AADHAAR_CARD", "PAN_CARD", "CREDIT_CARD", "VISA_CARD", "MASTER_CARD", "AMEX_CARD", "RUPAY_CARD", "MAESTRO_CARD", "VOTERID","BANK_ACCOUNT_NUMBER", "PHONE_NUMBER", "GST_NUMBER",   ],
    "US": [],
}